{% extends "./base.html" %}

{% block title %}{% if subtitle %}{{ subtitle }} | {% endif %}{{ title }} |
    {{ site_title|default:_('Django site admin') }}{% endblock %}

{% block branding %}
    <h1 id="site-name"><a href="{% url 'admin:index' %}">{{ site_header|default:_('OpenHub') }}</a></h1>
{% endblock %}

{% block nav-global %}

    <div id="top_menu"><a
            href="/hubs/">Hubs</a> | <a href="/hardwares/">Hardware</a> | <a
            href="/accessories/">Accessories</a></div>
{% endblock %}
